@@ -4,7 +4,7 @@
 #
 # A library for simple, DRY configuration of applications
 #
-# (C) 2009--2011 by Canonical Ltd.
+# (C) 2009--2013 by Canonical Ltd.
 # by John R. Lenton <john.lenton@canonical.com>
 # and Ricardo Kirkner <ricardo.kirkner@canonical.com>
 #
@@ -18,7 +18,8 @@
 AttributtedConfigParser lives here.
 """
 import re
-from ConfigParser import RawConfigParser
+
+from configglue._compat import RawConfigParser
 
 
 marker = object()
