@@ -4,7 +4,7 @@
 #
 # A library for simple, DRY configuration of applications
 #
-# (C) 2009--2011 by Canonical Ltd.
+# (C) 2009--2013 by Canonical Ltd.
 # by John R. Lenton <john.lenton@canonical.com>
 # and Ricardo Kirkner <ricardo.kirkner@canonical.com>
 #
@@ -15,10 +15,11 @@
 ###############################################################################
 
 """ TypedConfigParser lives here """
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import os
 
+from configglue._compat import text_type
 from . import parsers
 from .attributed import AttributedConfigParser
 
@@ -34,7 +35,7 @@ class TypedConfigParser(AttributedConfigParser):
                         'float': float,
                         'int': int,
                         'lines': parsers.lines,
-                        'unicode': unicode,
+                        'unicode': text_type,
                         'getenv': os.getenv,
                         None: lambda x: x}
 
