@@ -4,7 +4,7 @@
 #
 # A library for simple, DRY configuration of applications
 #
-# (C) 2009--2011 by Canonical Ltd.
+# (C) 2009--2013 by Canonical Ltd.
 # by John R. Lenton <john.lenton@canonical.com>
 # and Ricardo Kirkner <ricardo.kirkner@canonical.com>
 #
@@ -20,18 +20,17 @@ import copy
 import logging
 import os
 import re
-import string
 
-from ConfigParser import (
+from functools import reduce
+
+from ._compat import BaseConfigParser, text_type, string_types
+from ._compat import (
     DEFAULTSECT,
-    SafeConfigParser as BaseConfigParser,
     InterpolationMissingOptionError,
     NoOptionError,
     NoSectionError,
 )
 
-from configglue.schema import DictOption
-
 
 __all__ = [
     'SchemaValidationError',
@@ -40,6 +39,14 @@ __all__ = [
 
 CONFIG_FILE_ENCODING = 'utf-8'
 
+class NullHandler(logging.Handler):
+    def emit(self, record):
+        pass
+
+
+logger = logging.getLogger(__name__)
+logger.addHandler(NullHandler())
+
 
 class SchemaValidationError(Exception):
     """Exception class raised for any schema validation error."""
@@ -149,8 +156,8 @@ class SchemaConfigParser(BaseConfigParser, object):
             # structure validates, validate content
             self.parse_all()
 
-        except Exception, e:
-            errors.append(str(e))
+        except Exception as e:
+            errors.append(text_type(e))
             valid = False
 
         if report:
@@ -195,7 +202,7 @@ class SchemaConfigParser(BaseConfigParser, object):
             for option in options:
                 try:
                     value = self._interpolate(section, option, d[option], d)
-                except InterpolationMissingOptionError, e:
+                except InterpolationMissingOptionError as e:
                     # interpolation failed, because key was not found in
                     # section. try other sections before bailing out
                     value = self._interpolate_value(section, option)
@@ -237,7 +244,7 @@ class SchemaConfigParser(BaseConfigParser, object):
         """Like ConfigParser.read, but consider files we've already read."""
         if already_read is None:
             already_read = set()
-        if isinstance(filenames, basestring):
+        if isinstance(filenames, string_types):
             filenames = [filenames]
         read_ok = []
         for filename in filenames:
@@ -245,9 +252,9 @@ class SchemaConfigParser(BaseConfigParser, object):
             if path in already_read:
                 continue
             try:
-                fp = codecs.open(path, 'r', CONFIG_FILE_ENCODING)
+                fp = codecs.open(path, 'r', encoding=CONFIG_FILE_ENCODING)
             except IOError:
-                logging.warn(
+                logger.warn(
                     'File {0} could not be read. Skipping.'.format(path))
                 continue
             self._read(fp, path, already_read=already_read)
@@ -274,7 +281,7 @@ class SchemaConfigParser(BaseConfigParser, object):
             old_basedir, self._basedir = self._basedir, os.path.dirname(
                 fpname)
             includes = self.get('__main__', 'includes')
-            filenames = map(string.strip, includes)
+            filenames = [text_type.strip(x) for x in includes]
             self.read(filenames, already_read=already_read)
             self._basedir = old_basedir
 
@@ -294,7 +301,7 @@ class SchemaConfigParser(BaseConfigParser, object):
 
     def _update_location(self, old_sections, filename):
         # keep list of valid options to include locations for
-        option_names = map(lambda x: x.name, self.schema.options())
+        option_names = [x.name for x in self.schema.options()]
 
         # new values
         sections = self._sections
@@ -342,7 +349,7 @@ class SchemaConfigParser(BaseConfigParser, object):
 
             try:
                 value = option_obj.parse(value, **kwargs)
-            except ValueError, e:
+            except ValueError as e:
                 raise ValueError("Invalid value '%s' for %s '%s' in"
                     " section '%s'. Original exception was: %s" %
                     (value, option_obj.__class__.__name__, option,
@@ -373,10 +380,10 @@ class SchemaConfigParser(BaseConfigParser, object):
 
     def _extract_interpolation_keys(self, item):
         if isinstance(item, (list, tuple)):
-            keys = map(self._extract_interpolation_keys, item)
+            keys = [self._extract_interpolation_keys(x) for x in item]
             keys = reduce(set.union, keys, set())
         else:
-            keys = set(self._KEYCRE.findall(item))
+            keys = set(self._interpolation._KEYCRE.findall(item))
         # remove invalid key
         if '' in keys:
             keys.remove('')
@@ -384,7 +391,7 @@ class SchemaConfigParser(BaseConfigParser, object):
 
     def _get_interpolation_keys(self, section, option):
 
-        rawval = super(SchemaConfigParser, self).get(section, option, True)
+        rawval = super(SchemaConfigParser, self).get(section, option, raw=True)
         try:
             opt = self.schema.section(section).option(option)
             value = opt.parse(rawval, raw=True)
@@ -394,6 +401,10 @@ class SchemaConfigParser(BaseConfigParser, object):
         keys = self._extract_interpolation_keys(value)
         return rawval, keys
 
+    def _interpolate(self, *args, **kwargs):
+        """Helper method for transition to configparser."""
+        return self._interpolation.before_get(self, *args, **kwargs)
+
     def _interpolate_value(self, section, option):
         rawval, keys = self._get_interpolation_keys(section, option)
         if not keys:
@@ -423,24 +434,49 @@ class SchemaConfigParser(BaseConfigParser, object):
         # replace holders with values
         result = rawval % values
 
-        assert isinstance(result, basestring)
+        assert isinstance(result, string_types)
         return result
 
     def interpolate_environment(self, rawval, raw=False):
+        """Interpolate environment variables"""
         if raw:
             return rawval
 
-        # interpolate environment variables
+        # this allows both nested and mutliple environment variable
+        # interpolation in a single value
         pattern = re.sub(r'\${([A-Z_]+)}', r'%(\1)s', rawval)
         pattern = re.sub(r'\$([A-Z_]+)', r'%(\1)s', pattern)
 
+        # counter to protect against infinite loops
+        num_interpolations = 0
+        # save simple result in case we need it
+        simple_pattern = pattern
+
+        # handle complex case of env vars with defaults
+        env = os.environ.copy()
+        env_re = re.compile(r'\${(?P<name>[A-Z_]+):-(?P<default>.*?)}')
+        match = env_re.search(pattern)
+        while match and num_interpolations < 50:
+            groups = match.groupdict()
+            name = groups['name']
+            pattern = pattern.replace(match.group(), '%%(%s)s' % name)
+            if name not in env and groups['default'] is not None:
+                # interpolate defaults as well to allow ${FOO:-$BAR}
+                env[name] = groups['default'] % os.environ
+
+            num_interpolations += 1
+            match = env_re.search(pattern)
+
+        if num_interpolations >= 50:
+            # blown loop, restore earlier simple interpolation
+            pattern = simple_pattern
+
         keys = self._extract_interpolation_keys(pattern)
         if not keys:
             # interpolation keys are not valid
             return rawval
 
-        interpolated = pattern % os.environ
-        return interpolated
+        return pattern % env
 
     def _get_default(self, section, option):
         # cater for 'special' sections
@@ -477,20 +513,20 @@ class SchemaConfigParser(BaseConfigParser, object):
                 pass
             # value is defined entirely in current section
             value = super(SchemaConfigParser, self).get(section, option,
-                                                        raw, vars)
-        except InterpolationMissingOptionError, e:
+                                                        raw=raw, vars=vars)
+        except InterpolationMissingOptionError as e:
             # interpolation key not in same section
             value = self._interpolate_value(section, option)
             if value is None:
                 # this should be a string, so None indicates an error
                 raise e
-        except (NoSectionError, NoOptionError), e:
+        except (NoSectionError, NoOptionError) as e:
             # option not found in config, try to get its default value from
             # schema
             value = self._get_default(section, option)
 
         # interpolate environment variables
-        if isinstance(value, basestring):
+        if isinstance(value, string_types):
             try:
                 value = self.interpolate_environment(value, raw=raw)
                 if parse:
@@ -551,11 +587,9 @@ class SchemaConfigParser(BaseConfigParser, object):
 
         """
         if fp is not None:
-            if isinstance(fp, basestring):
-                fp = open(fp, 'w')
-            # write to a specific file
-            encoded_fp = codecs.getwriter(CONFIG_FILE_ENCODING)(fp)
-            self.write(encoded_fp)
+            if isinstance(fp, string_types):
+                fp = codecs.open(fp, 'w', encoding=CONFIG_FILE_ENCODING)
+            self.write(fp)
         else:
             # write to the original files
             for filename, sections in self._dirty.items():
@@ -580,7 +614,8 @@ class SchemaConfigParser(BaseConfigParser, object):
                         parser.set(section, option, value)
 
                 # write to new file
-                parser.write(open("%s.new" % filename, 'w'))
+                parser.write(codecs.open("%s.new" % filename, 'w',
+                                         encoding=CONFIG_FILE_ENCODING))
                 # rename old file
                 if os.path.exists(filename):
                     os.rename(filename, "%s.old" % filename)
