@@ -4,7 +4,7 @@
 #
 # A library for simple, DRY configuration of applications
 #
-# (C) 2009--2011 by Canonical Ltd.
+# (C) 2009--2013 by Canonical Ltd.
 # by John R. Lenton <john.lenton@canonical.com>
 # and Ricardo Kirkner <ricardo.kirkner@canonical.com>
 #
@@ -13,15 +13,16 @@
 # For bug reports, support, and new releases: http://launchpad.net/configglue
 #
 ###############################################################################
+from __future__ import unicode_literals
 
 import json
-from ConfigParser import (
-    NoSectionError,
-    NoOptionError,
-)
 from copy import deepcopy
 from inspect import getmembers
 
+from ._compat import text_type, string_types
+from ._compat import NoSectionError, NoOptionError
+
+
 
 __all__ = [
     'BoolOption',
@@ -156,6 +157,9 @@ class Schema(object):
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __hash__(self):
+        return id(self)
+
     def is_valid(self):
         """Return whether the schema has a valid structure."""
         explicit_default_section = isinstance(getattr(self, '__main__', None),
@@ -186,7 +190,7 @@ class Schema(object):
         To get options from the default section, specify section='__main__'
 
         """
-        if isinstance(section, basestring):
+        if isinstance(section, string_types):
             section = self.section(section)
         if section is None:
             options = []
@@ -222,6 +226,9 @@ class Section(object):
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __hash__(self):
+        return id(self)
+
     def __repr__(self):
         if self.name:
             name = " %s" % self.name
@@ -316,6 +323,9 @@ class Option(object):
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __hash__(self):
+        return id(self)
+
     def __repr__(self):
         extra = ' raw' if self.raw else ''
         extra += ' fatal' if self.fatal else ''
@@ -332,7 +342,7 @@ class Option(object):
     def _get_default(self):
         return None
 
-    def parse(self, value):
+    def parse(self, value, raw=False):
         """Parse the given value."""
         raise NotImplementedError()
 
@@ -341,7 +351,7 @@ class Option(object):
 
     def to_string(self, value):
         """Return a string representation of the value."""
-        return str(value)
+        return text_type(value)
 
 
 class BoolOption(Option):
@@ -432,6 +442,9 @@ class ListOption(Option):
 
         return equal
 
+    def __hash__(self):
+        return id(self)
+
     def _get_default(self):
         return []
 
@@ -501,6 +514,9 @@ class StringOption(Option):
 
         return equal
 
+    def __hash__(self):
+        return id(self)
+
     def _get_default(self):
         return '' if not self.null else None
 
@@ -514,7 +530,7 @@ class StringOption(Option):
             result = value
         elif self.null:
             result = None if value in (None, 'None') else value
-        elif isinstance(value, basestring):
+        elif isinstance(value, string_types):
             result = value
         else:
             result = repr(value)
@@ -526,7 +542,7 @@ class StringOption(Option):
         return value
 
     def validate(self, value):
-        return (self.null and value is None) or isinstance(value, basestring)
+        return (self.null and value is None) or isinstance(value, string_types)
 
 
 class TupleOption(Option):
@@ -552,6 +568,9 @@ class TupleOption(Option):
 
         return equal
 
+    def __hash__(self):
+        return id(self)
+
     def _get_default(self):
         return ()
 
@@ -620,6 +639,9 @@ class DictOption(Option):
 
         return equal
 
+    def __hash__(self):
+        return id(self)
+
     def _get_default(self):
         default = {}
         for key, value in self.spec.items():
@@ -663,12 +685,13 @@ class DictOption(Option):
                 # parse it using the default item parser
                 option = self.item
 
-            # parse option
-            kwargs = {}
-            if option.require_parser:
-                kwargs['parser'] = parser
-            if not raw:
-                value = option.parse(value, **kwargs)
+            if not option.validate(value):
+                # parse option
+                kwargs = {}
+                if option.require_parser:
+                    kwargs['parser'] = parser
+                if not raw:
+                    value = option.parse(value, **kwargs)
             result[key] = value
 
         # fill in missing items with default values
@@ -682,7 +705,7 @@ class DictOption(Option):
                     if not raw:
                         value = option.default
                     else:
-                        value = unicode(option.default)
+                        value = text_type(option.default)
                     result[key] = value
         return result
 
