@@ -5,7 +5,7 @@
 #
 # A library for simple, DRY configuration of applications
 #
-# (C) 2009--2011 by Canonical Ltd.
+# (C) 2009--2013 by Canonical Ltd.
 # by John R. Lenton <john.lenton@canonical.com>
 # and Ricardo Kirkner <ricardo.kirkner@canonical.com>
 #
@@ -14,15 +14,14 @@
 # For bug reports, support, and new releases: http://launchpad.net/configglue
 #
 ###############################################################################
+from __future__ import unicode_literals
 
 import textwrap
 import unittest
-from ConfigParser import (
-    NoOptionError,
-    NoSectionError,
-)
-from StringIO import StringIO
+from io import BytesIO
 
+from configglue._compat import text_type
+from configglue._compat import NoOptionError, NoSectionError
 from configglue.parser import (
     SchemaConfigParser,
     SchemaValidationError,
@@ -389,19 +388,19 @@ class TestIntOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls()
 
-        config = StringIO("[__main__]\nfoo = 42")
+        config = BytesIO(b"[__main__]\nfoo = 42")
         expected_values = {'__main__': {'foo': 42}}
         schema = MySchema()
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertEqual(parser.values(), expected_values)
 
-        config = StringIO("[__main__]\nfoo =")
+        config = BytesIO(b"[__main__]\nfoo =")
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertRaises(ValueError, parser.values)
 
-        config = StringIO("[__main__]\nfoo = bla")
+        config = BytesIO(b"[__main__]\nfoo = bla")
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertRaises(ValueError, parser.values)
@@ -435,24 +434,24 @@ class TestBoolOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls()
 
-        config = StringIO("[__main__]\nfoo = Yes")
+        config = BytesIO(b"[__main__]\nfoo = Yes")
         expected_values = {'__main__': {'foo': True}}
         schema = MySchema()
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertEqual(parser.values(), expected_values)
 
-        config = StringIO("[__main__]\nfoo = tRuE")
+        config = BytesIO(b"[__main__]\nfoo = tRuE")
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertEqual(parser.values(), expected_values)
 
-        config = StringIO("[__main__]\nfoo =")
+        config = BytesIO(b"[__main__]\nfoo =")
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertRaises(ValueError, parser.values)
 
-        config = StringIO("[__main__]\nfoo = bla")
+        config = BytesIO(b"[__main__]\nfoo = bla")
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertRaises(ValueError, parser.values)
@@ -486,7 +485,7 @@ class TestListOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls(item=IntOption())
 
-        config = StringIO("[__main__]\nfoo = 42\n 43\n 44")
+        config = BytesIO(b"[__main__]\nfoo = 42\n 43\n 44")
         expected_values = {'__main__': {'foo': [42, 43, 44]}}
         schema = MySchema()
         parser = SchemaConfigParser(schema)
@@ -497,7 +496,7 @@ class TestListOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls(item=IntOption(), parse_json=False)
 
-        config = StringIO("[__main__]\nfoo = 42\n 43\n 44")
+        config = BytesIO(b"[__main__]\nfoo = 42\n 43\n 44")
         expected_values = {'__main__': {'foo': [42, 43, 44]}}
         schema = MySchema()
         parser = SchemaConfigParser(schema)
@@ -508,7 +507,7 @@ class TestListOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls(item=IntOption(), parse_json=False)
 
-        config = StringIO("[__main__]\nfoo = [42, 43, 44]")
+        config = BytesIO(b"[__main__]\nfoo = [42, 43, 44]")
         schema = MySchema()
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
@@ -518,7 +517,7 @@ class TestListOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls(item=IntOption())
 
-        config = StringIO("[__main__]\nfoo = [42, 43, 44]")
+        config = BytesIO(b"[__main__]\nfoo = [42, 43, 44]")
         expected_values = {'__main__': {'foo': [42, 43, 44]}}
         schema = MySchema()
         parser = SchemaConfigParser(schema)
@@ -529,7 +528,7 @@ class TestListOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls(item=IntOption())
 
-        config = StringIO('[__main__]\nfoo = 1, 2, 3')
+        config = BytesIO(b'[__main__]\nfoo = 1, 2, 3')
         schema = MySchema()
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
@@ -539,7 +538,7 @@ class TestListOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls(item=IntOption())
 
-        config = StringIO('[__main__]\nfoo = {"foo": "bar"}')
+        config = BytesIO(b'[__main__]\nfoo = {"foo": "bar"}')
         schema = MySchema()
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
@@ -551,7 +550,7 @@ class TestListOption(unittest.TestCase):
             foo = self.cls(item=BoolOption())
 
         schema = MySchema()
-        config = StringIO("[__main__]\nfoo = tRuE\n No\n 0\n 1")
+        config = BytesIO(b"[__main__]\nfoo = tRuE\n No\n 0\n 1")
         expected_values = {'__main__': {'foo': [True, False, False, True]}}
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
@@ -563,7 +562,7 @@ class TestListOption(unittest.TestCase):
             foo = self.cls(item=BoolOption())
 
         schema = MySchema()
-        config = StringIO("[__main__]\nfoo =")
+        config = BytesIO(b"[__main__]\nfoo =")
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         expected_values = {'__main__': {'foo': []}}
@@ -575,12 +574,12 @@ class TestListOption(unittest.TestCase):
             foo = self.cls(item=BoolOption())
 
         schema = MySchema()
-        config = StringIO("[__main__]\nfoo = bla")
+        config = BytesIO(b"[__main__]\nfoo = bla")
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertRaises(ValueError, parser.values)
 
-        config = StringIO("[__main__]\nfoo = True\n bla")
+        config = BytesIO(b"[__main__]\nfoo = True\n bla")
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertRaises(ValueError, parser.values)
@@ -596,7 +595,7 @@ class TestListOption(unittest.TestCase):
             foo = self.cls(item=StringOption(), remove_duplicates=True)
 
         schema = MySchema()
-        config = StringIO("[__main__]\nfoo = bla\n blah\n bla")
+        config = BytesIO(b"[__main__]\nfoo = bla\n blah\n bla")
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertEquals({'__main__': {'foo': ['bla', 'blah']}},
@@ -608,7 +607,7 @@ class TestListOption(unittest.TestCase):
             foo = self.cls(item=DictOption(), remove_duplicates=True)
 
         schema = MyOtherSchema()
-        config = StringIO("[__main__]\nfoo = bla\n bla\n[bla]\nbar = baz")
+        config = BytesIO(b"[__main__]\nfoo = bla\n bla\n[bla]\nbar = baz")
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertEquals({'__main__': {'foo': [{'bar': 'baz'}]}},
@@ -661,7 +660,8 @@ class TestListOption(unittest.TestCase):
     def test_to_string_when_no_json(self):
         option = ListOption(parse_json=False)
         result = option.to_string(['1', '2', '3'])
-        self.assertEqual(result, "['1', '2', '3']")
+        expected = [text_type(x) for x in [1, 2, 3]]
+        self.assertEqual(result, text_type(expected))
 
 
 class TestTupleOption(unittest.TestCase):
@@ -683,7 +683,7 @@ class TestTupleOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls()
 
-        config = StringIO('[__main__]\nfoo=1,2,3,4')
+        config = BytesIO(b'[__main__]\nfoo=1,2,3,4')
         expected_values = {'__main__': {'foo': ('1', '2', '3', '4')}}
         parser = SchemaConfigParser(MySchema())
         parser.readfp(config)
@@ -694,19 +694,19 @@ class TestTupleOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls(length=4)
 
-        config = StringIO('[__main__]\nfoo = 1, 2, 3, 4')
+        config = BytesIO(b'[__main__]\nfoo = 1, 2, 3, 4')
         expected_values = {'__main__': {'foo': ('1', '2', '3', '4')}}
         schema = MySchema()
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertEqual(parser.values(), expected_values)
 
-        config = StringIO('[__main__]\nfoo = 1, 2, 3')
+        config = BytesIO(b'[__main__]\nfoo = 1, 2, 3')
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertRaises(ValueError, parser.values)
 
-        config = StringIO('[__main__]\nfoo = ')
+        config = BytesIO(b'[__main__]\nfoo = ')
         parser = SchemaConfigParser(schema)
         parser.readfp(config)
         self.assertRaises(ValueError, parser.values)
@@ -763,7 +763,7 @@ class TestDictOption(unittest.TestCase):
         class MySchema(Schema):
             foo = self.cls(item=self.cls())
 
-        config = StringIO("""
+        config = BytesIO(b"""
 [__main__]
 foo=dict1
 [dict1]
@@ -788,7 +788,7 @@ baz=42
                 'bla': BoolOption(),
             })
 
-        config = StringIO("""[__main__]
+        config = BytesIO(b"""[__main__]
 foo = mydict
 [mydict]
 bar=baz
@@ -813,7 +813,7 @@ bla=Yes
                 'bla': BoolOption(),
             }, parse_json=False)
 
-        config = StringIO("""[__main__]
+        config = BytesIO(b"""[__main__]
 foo = mydict
 [mydict]
 bar=baz
@@ -838,13 +838,13 @@ bla=Yes
                 'bla': BoolOption(),
             })
 
-        config = StringIO(textwrap.dedent("""
+        config = BytesIO(textwrap.dedent("""
             [__main__]
             foo = {
                 "bar": "baz",
                 "baz": "42",
                 "bla": "Yes"}
-            """))
+            """).encode('utf-8'))
         expected_values = {
             '__main__': {
                 'foo': {'bar': 'baz', 'baz': 42, 'bla': True}}}
@@ -863,10 +863,10 @@ bla=Yes
                 'bla': BoolOption(),
             })
 
-        config = StringIO(textwrap.dedent("""
+        config = BytesIO(textwrap.dedent("""
             [__main__]
             foo = {'bar': 23}
-            """))
+            """).encode('utf-8'))
 
         schema = MySchema()
         parser = SchemaConfigParser(schema)
@@ -882,10 +882,10 @@ bla=Yes
                 'bla': BoolOption(),
             })
 
-        config = StringIO(textwrap.dedent("""
+        config = BytesIO(textwrap.dedent("""
             [__main__]
             foo = [1, 2, 3]
-            """))
+            """).encode('utf-8'))
 
         schema = MySchema()
         parser = SchemaConfigParser(schema)
@@ -901,13 +901,13 @@ bla=Yes
                 'bla': BoolOption(),
             }, parse_json=False)
 
-        config = StringIO(textwrap.dedent("""
+        config = BytesIO(textwrap.dedent("""
             [__main__]
             foo = {
                 "bar": "baz",
                 "baz": "42",
                 "bla": "Yes"}
-            """))
+            """).encode('utf-8'))
 
         schema = MySchema()
         parser = SchemaConfigParser(schema)
@@ -923,7 +923,7 @@ bla=Yes
                 'bla': BoolOption(),
             })
 
-        config = StringIO("""[__main__]
+        config = BytesIO(b"""[__main__]
 foo = mydict
 [mydict]
 baz=42
@@ -936,12 +936,54 @@ baz=42
         parsed = schema.foo.parse('mydict', parser, True)
         self.assertEqual(parsed, expected)
 
+    def test_parse_json_raw_with_interpolation_marks(self):
+        """Test DictOption parse json using raw=True when data has interpolation marks."""
+        class MySchema(Schema):
+            class logging(Section):
+                formatters = self.cls(raw=True, item=self.cls())
+
+        config = BytesIO(textwrap.dedent("""
+            [logging]
+            formatters = {"sample": {"format": "%(name)s"}}
+            """).encode('utf-8'))
+        expected = {'sample': {'format': '%(name)s'}}
+
+        schema = MySchema()
+        parser = SchemaConfigParser(schema)
+        parser.readfp(config)
+        parsed = parser.values('logging')['formatters']
+        self.assertEqual(parsed, expected)
+
+    def test_parse_no_json_raw_with_interpolation_marks(self):
+        """Test DictOption parse non-json using raw=True when data has interpolation marks."""
+        class MySchema(Schema):
+            class logging(Section):
+                formatters = self.cls(raw=True, item=self.cls())
+
+        config = BytesIO(textwrap.dedent("""
+            [logging]
+            formatters = logging_formatters
+
+            [logging_formatters]
+            sample = sample_formatter
+
+            [sample_formatter]
+            format = %%(name)s
+            """).encode('utf-8'))
+        expected = {'sample': {'format': '%(name)s'}}
+
+        schema = MySchema()
+        parser = SchemaConfigParser(schema)
+        parser.readfp(config)
+        parsed = parser.values('logging')['formatters']
+        self.assertEqual(parsed, expected)
+
     def test_parse_invalid_key_in_parsed(self):
         """Test DictOption parse with an invalid key in the config."""
         class MySchema(Schema):
             foo = self.cls(spec={'bar': IntOption()})
 
-        config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbaz=2")
+        config = BytesIO(b"[__main__]\nfoo=mydict\n[mydict]\nbaz=2")
         expected_values = {'__main__': {'foo': {'bar': 0, 'baz': '2'}}}
         parser = SchemaConfigParser(MySchema())
         parser.readfp(config)
@@ -954,7 +996,7 @@ baz=42
                 'bar': IntOption(),
                 'baz': IntOption(fatal=True)})
 
-        config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=2")
+        config = BytesIO(b"[__main__]\nfoo=mydict\n[mydict]\nbar=2")
         parser = SchemaConfigParser(MySchema())
         parser.readfp(config)
         self.assertRaises(ValueError, parser.parse_all)
@@ -968,7 +1010,7 @@ baz=42
         class MySchema(Schema):
             foo = self.cls(spec={'bar': IntOption()})
 
-        config = StringIO("[__main__]\nfoo=mydict\n[mydict]")
+        config = BytesIO(b"[__main__]\nfoo=mydict\n[mydict]")
         expected_values = {'__main__': {'foo': {'bar': 0}}}
         parser = SchemaConfigParser(MySchema())
         parser.readfp(config)
@@ -978,7 +1020,7 @@ baz=42
         class MySchema(Schema):
             foo = self.cls()
 
-        config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=2")
+        config = BytesIO(b"[__main__]\nfoo=mydict\n[mydict]\nbar=2")
         expected_values = {'__main__': {'foo': {'bar': '2'}}}
         parser = SchemaConfigParser(MySchema())
         parser.readfp(config)
@@ -990,7 +1032,7 @@ baz=42
             foo = self.cls(
                       item=self.cls(
                           item=IntOption()))
-        config = StringIO("""
+        config = BytesIO(b"""
 [__main__]
 foo = mydict
 [mydict]
@@ -1009,7 +1051,7 @@ wham=42
             spec = {'bar': IntOption()}
             foo = self.cls(spec=spec, strict=True)
 
-        config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=2")
+        config = BytesIO(b"[__main__]\nfoo=mydict\n[mydict]\nbar=2")
         expected_values = {'__main__': {'foo': {'bar': 2}}}
         parser = SchemaConfigParser(MySchema())
         parser.readfp(config)
@@ -1022,7 +1064,7 @@ wham=42
                     'baz': IntOption()}
             foo = self.cls(spec=spec, strict=True)
 
-        config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=2")
+        config = BytesIO(b"[__main__]\nfoo=mydict\n[mydict]\nbar=2")
         expected_values = {'__main__': {'foo': {'bar': 2, 'baz': 0}}}
         parser = SchemaConfigParser(MySchema())
         parser.readfp(config)
@@ -1034,7 +1076,7 @@ wham=42
             spec = {'bar': IntOption()}
             foo = self.cls(spec=spec, strict=True)
 
-        config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=2\nbaz=3")
+        config = BytesIO(b"[__main__]\nfoo=mydict\n[mydict]\nbar=2\nbaz=3")
         parser = SchemaConfigParser(MySchema())
         parser.readfp(config)
         self.assertRaises(ValueError, parser.parse_all)
@@ -1074,7 +1116,7 @@ wham=42
     def test_to_string_when_no_json(self):
         option = DictOption(parse_json=False)
         result = option.to_string({'foo': '1'})
-        self.assertEqual(result, str({'foo': '1'}))
+        self.assertEqual(result, text_type({'foo': '1'}))
 
 
 class TestListOfDictOption(unittest.TestCase):
@@ -1088,7 +1130,7 @@ class TestListOfDictOption(unittest.TestCase):
                     'bla': BoolOption(),
                 }))
 
-        config = StringIO("""[__main__]
+        config = BytesIO(b"""[__main__]
 foo = mylist0
       mylist1
 [mylist0]
@@ -1125,7 +1167,7 @@ class TestDictWithDicts(unittest.TestCase):
         class MySchema(Schema):
             foo = DictOption(spec=spec)
 
-        config = StringIO("""[__main__]
+        config = BytesIO(b"""[__main__]
 foo = outerdict
 [outerdict]
 options = innerdict
@@ -1152,19 +1194,19 @@ class TestListOfTuples(unittest.TestCase):
         self.parser = SchemaConfigParser(schema)
 
     def test_parse_list_of_tuples(self):
-        config = StringIO('[__main__]\nfoo = a, b, c\n      d, e, f')
+        config = BytesIO(b'[__main__]\nfoo = a, b, c\n      d, e, f')
         expected_values = {
             '__main__': {'foo': [('a', 'b', 'c'), ('d', 'e', 'f')]}}
         self.parser.readfp(config)
         self.assertEqual(self.parser.values(), expected_values)
 
     def test_parse_wrong_tuple_size(self):
-        config = StringIO('[__main__]\nfoo = a, b, c\n      d, e')
+        config = BytesIO(b'[__main__]\nfoo = a, b, c\n      d, e')
         self.parser.readfp(config)
         self.assertRaises(ValueError, self.parser.values)
 
     def test_parse_empty_tuple(self):
-        config = StringIO('[__main__]\nfoo=()')
+        config = BytesIO(b'[__main__]\nfoo=()')
         expected_values = {'__main__': {'foo': [()]}}
         self.parser.readfp(config)
         self.assertEqual(self.parser.values(), expected_values)
@@ -1289,6 +1331,6 @@ class MultiSchemaTestCase(unittest.TestCase):
         try:
             merge(SchemaA, SchemaB)
             self.fail('SchemaValidationError not raised.')
-        except SchemaValidationError, e:
-            self.assertEqual(str(e),
+        except SchemaValidationError as e:
+            self.assertEqual(text_type(e),
                 "Conflicting option '__main__.foo' while merging schemas.")
